/*
 * uni_create.c
 *
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "libuni.h"

/* uni_create : Returns 0 on successful completion. */
int
uni_create (double *xdata, double *ydata, long int xelems, long int yelems,
	                struct unifile *ufile)
{
	double *xdatad, *ydatad;
	int i;
	int retval;
	int place_in = 0;
	int n_outbuf = 0;
	long int m_out;
	char tmpstr[50];
	double xlims[4];
	double dx_min, dx_max, maxratio = 1.0, dx_last = 1.0;
	double dx0, x1max, x1, xlast, x1min;
	double ymin, ymax, yav;
	double zlocal[2][4];
	double flocal[4];
	double qcoefs[2][3];
	double x_nexttolastin;
	double x_out;

	long int zelems = 0L;


	xdatad = (double *) malloc (sizeof (double) * (xelems + 2));
	ydatad = (double *) malloc (sizeof (double) * (yelems + 2));


	for (i = 0; i < xelems; i++)
		{
			xdatad[i+1] = xdata[i];
		}

	for (i = 0; i < yelems; i++)
		{
			ydatad[i+1] = ydata[i];
		}

	xlims[0] = xdata[0];
	xlims[1] = xdata[1];

	/* We should probably ensure there are at least 4 elements */
	xlims[2] = xdata[xelems-2];
	xlims[3] = xdata[xelems-1];

	/* Initialize dx_min and dx_max */
	dx_min = xlims[1] - xlims[0];
	dx_max = dx_min;

		/* xelems - 1 so we don't overrun the array */
	for (i = 0; i < xelems - 1; i++)
		{	
			double dx_current;
			dx_current = xdata[i+1] - xdata[i];

			if (dx_current > dx_max)
				dx_max = dx_current;

			if (dx_current < dx_min)
				dx_min = dx_current;

			/* If i is nonzero, it means that we have a dx_last
			 * to work with
			 */

			if (i)
				{
					double temp;
					temp = dx_current/dx_last;
					if (temp > maxratio)
						maxratio = temp;
					temp = dx_last/dx_current;
					if (temp > maxratio)
						maxratio = temp;
				}

			dx_last = dx_current;
		}


	/* This is the "default" value for DX0 */
	dx0 = (0.2*dx_min);

	/* This is really, really weird, but it seems to be the only way
	 * to get the values right */
	sprintf(tmpstr, "%.2le", dx0);
	sscanf(tmpstr, "%le", &dx0);

	for (i = 0; i < 4000; i++)
		{
			x1min = xlims[0] + 0.0004*i*dx0;
			sprintf (tmpstr, "                   ");
			sprintf (tmpstr, "%.7lg", x1min);
			sscanf (tmpstr, "%le", &x1min);
			if (x1min > xlims[0])
				i = 5000;
		}
	if (i < 5000)
		{
			#ifdef _DEBUG
			fprintf (stderr, "ERROR: uni_create.c - Floating point precision error\n");
			#endif
			return 1;
		}

	/* Setting X1MAX - where is the 0.7 coming from? */
	x1max = xlims[2] - 0.7*dx0;

  #ifdef _DEBUG
  fprintf (stderr, "uni_create.c - XN is %lf\n", x1max);
  #endif

	/* Again, paranoia that the interval is too small to sum using the
	 * limited precision of our doubles
	 */
	for (i = 0; i < 4000; i++)
		{
			x1 = 0.625*xlims[0] + 0.375*xlims[1] + .0002*i*dx0;
			sprintf(tmpstr, "                ");
			sprintf(tmpstr, "%.7lg", x1);
			sscanf(tmpstr, "%le", &x1);
			if (x1 > xlims[0])
				i = 5000;
		}
	if (i < 5000)
		{
			#ifdef _DEBUG
			fprintf (stderr, "ERROR: uni_create.c - Floating point precision error\n");
			#endif
			return 1;
		}
	else
		{
			int temp;
			temp = (int) (0.75*xlims[3] + 0.25*xlims[2] - x1)/dx0;
			xlast = x1 + temp * dx0;
      #ifdef _DEBUG
      fprintf (stderr, "uni_create.c - xlast is %lf\n", xlast);
      #endif

			/* Number of 'elements' in the output file, I think */
			zelems = temp + 1L;
		}

	/* Initialize min max avg */
	ymin = ydata[0];
	ymax = ymin;
	yav = 0.0;

	/* Find the min, max, and average in F(X) data */
	for (i = 0; i < yelems; i++)
		{
			double temp;
			temp = ydata[i];
			if (temp < ymin)
				ymin = temp;
			if (temp > ymax)
				ymax = temp;
			yav += temp;
		}
	yav /= yelems;

	/* In theory, all the parameters for the UNIFSPEC format are collected. */

	/* Now, we need to open a new file, write its header, and then 
	 * write the data to it! Simple.. right?
	 */

		x_nexttolastin = xdatad[2];
	
	/* Now, the actual algorithm to write the data.. begins! */
	xdatad[0] = 2.0 * xdatad[1] - xdatad[2];
	
	/* Extrapolate this value below */
	ydatad[0] = ydatad[1];

	for (i = 0; i < 4; i++)
		{
			double temp;
			temp = xdatad[i];
			zlocal[0][i] = temp - xdatad[2];
			flocal[i] = ydatad[i];
		}

	retval = qfit (&zlocal[0][1], &flocal[1], &qcoefs[0][0]);
	
	if (retval) /* Normal result */
		{
			double temp;
			temp = zlocal[0][0];
			flocal[0] = (qcoefs[0][2]*temp + qcoefs[0][1])*temp + qcoefs[0][0];
			ydatad[0] = flocal[0];
		}
	else
		{
			#ifdef _DEBUG
			fprintf (stderr, "ERROR: uni_create.c - qfit returned with 0\n");
			#endif
			return 2;
		}

	xdatad[xelems+1] = 2.0*xdatad[xelems] - xdatad[xelems-1];
	ydatad[yelems+1] = 2.0*ydatad[yelems] - ydatad[yelems-1];

	for (i = 0; i < 4; i++)
		{
			int j;
			double temp;
			j = yelems - 2 + i;
			temp = xdatad[j];
			zlocal[0][i] = temp - xdatad[xelems-1];
			flocal[i] = ydatad[j];
		}

	retval = qfit (&zlocal[0][0],&flocal[0],&qcoefs[0][0]);

	if (retval) /* Normal result */
		{
			double temp;
			temp = zlocal[0][3];
			flocal[3] = (qcoefs[0][2]*temp + qcoefs[0][1])*temp + qcoefs[0][0];
			ydatad[xelems+1] = flocal[3];
		}
	else
		{
			#ifdef _DEBUG
			fprintf (stderr, "ERROR: uni_create.c - qfit returned with 0\n");
			#endif
			return 2;
		}
			
	place_in = 0; n_outbuf = 0;

		ufile[0].num_elems = zelems;
		ufile[0].data = (double *) malloc (sizeof (double) * zelems);
		ufile[0].dx = dx0;
		ufile[0].x1 = x1;

		for (m_out = 1; m_out <= zelems; m_out++)
		{
			double eetemp;
			eetemp = m_out - 1;
			/* Choose next value of X for output list of F(X)'s */
			x_out = x1 + (eetemp)*dx0;
			
			while (xdatad[place_in+1] <= x_out)
				{
					++place_in;
					if (place_in < zelems-1) /* Not end of data */
						{
							/* Get two sets of interpolation coefs */
							double aa, bb;
							aa = xdatad[place_in];
							bb = xdatad[place_in + 1];
							for (i = 0; i <= 3; i++)
								{
									int k;
									double cc;
									k = place_in + i - 1;
									flocal[i] = ydatad[k];
									cc = xdatad[k];
									zlocal[0][i] = cc - aa;
									zlocal[1][i] = cc - bb;
								}
							retval = qfit(&zlocal[0][0],&flocal[0],&qcoefs[0][0]);
							if (!retval)
								{	
									#ifdef _DEBUG
									fprintf (stderr, "ERROR: uni_create.c - qfit returned with 0\n");
									#endif
									return 2;
								}
							retval = qfit(&zlocal[1][1],&flocal[1],&qcoefs[1][0]);
							if (!retval)
								{	
									#ifdef _DEBUG
									fprintf (stderr, "ERROR: uni_create.c - qfit returned with 0\n");
									#endif
									return 2;
								}
						}
					else 
						{
							#ifdef _DEBUG
							fprintf (stderr, "ERROR: uni_create.c - overshot data\n");
							#endif
							return 3;
						}
				}

			/* now, we do some work! */

			if (1)
				{
					double aa, bb, cc, dd, ee, ff, gg, hh;
					aa = xdatad[place_in];
					bb = x_out - aa;
					cc = (qcoefs[0][2]*bb + qcoefs[0][1])*bb + qcoefs[0][0];
					dd = xdatad[place_in+1];
					ee = x_out - dd;
					ff = (qcoefs[1][2]*ee + qcoefs[1][1])*ee + qcoefs[1][0];
					gg = dd - aa;
					if (gg == 0.0) /* should never happen */
						{
							#ifdef _DEBUG
							fprintf (stderr, "ERROR: uni_create.c - Strange zero interval\n");
							#endif
							return 3;
						}
					else /* normal case */
						{
							double out[1];
							hh = (bb*ff - ee*cc)/gg; /* weighted av of 2 quad interp */
							out[0] = hh;
							ufile[0].data[m_out-1] = out[0];
							++n_outbuf;
						}
				}

		}
	/* I think we're done.. maybe. */	
  ufile->header = NULL;
  ufile->header_lines = 0;
  uni_new_header (ufile);
	uni_calc_meta (ufile);
	free( xdatad );	/* we need to free this, right? */
	free( ydatad ); /* we need to free this, right? */
	return 0;
}

int
qfit (double *x, double *f, double *coef)
{
	int i, retval;
	double d, s, det;
	double x0, x2;
	double f0, f1, f2;

	for (i = 0; i < 3; i++)
		coef[i] = 0.0;

	retval = 0;

	x0 = x[0];
	x2 = x[2];

	f0 = f[0];
	f1 = f[1];
	f2 = f[2];

	d = x0 - x2;
	s = x0 + x2;
	det = x0*d*x2;

	if (det*det > 1.e-200)
		{
			coef[0] = f1;
			coef[1] = (x0*x0*f2 - d*s*f1 - (x2*x2)*f0)/det;
			coef[2] = (x2*f0 + d*f1 - x0*f2)/det;
			retval = 1;
		}

	return retval;
}
