/*
 * uni_create_header.c
 *
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "libuni.h"

/* uni_create_header : Returns 0 on successful completion. */
int
uni_create_header (char **header, long int *bytes, struct unifile *ufile)
{
	int i, stridx, result;
	int lines;
	char buf[1024];

	stridx = 0;
	bytes[0] = 0;

	/* We need to add two reserved keys to the header: ^ and X1,DX */
	sprintf (buf, "%.7lg %.4lg", ufile[0].x1, ufile[0].dx);
	result = uni_writekey (buf, "X1,DX", ufile);

	lines = ufile[0].header_lines;
	
	
	/* Loop through header lines, counting bytes */
	for (i = 0; i < lines; i++)
		{
			if (ufile[0].header[i] == NULL)
				{
					/* ERROR: NULL header, too many header_lines? ERROR CODE 2 */
					#ifdef _DEBUG
					fprintf (stderr, "ERROR: uni_create_header - NULL header at index %d\n", i);
					#endif
					return 2;
				}
			bytes[0] += strlen(ufile[0].header[i])+2; /* Data in header + CR + LF */
		}

	/* We need four more bytes for the #^ sentinel. */

	*bytes += 4;

	*header = (char *) malloc (sizeof (char) * (*bytes));

	if (*(header) == NULL)
		{
			/* ERROR: Unable to allocate memory for header. ERROR CODE 1 */
			#ifdef _DEBUG
			fprintf (stderr, "ERROR: uni_create_header - Unable to allocate memory for header.\n");
			#endif
			return 1;
		}

	/* Loop through header lines, replacing the '\0' character with CR-LFs */
	for (i = 0; i < lines; i++)
		{
			int linelen;

			linelen = strlen (ufile[0].header[i]);
			strncpy (header[0] + stridx, ufile[0].header[i], linelen);
			stridx += linelen;
			header[0][stridx] = 13;
			header[0][stridx+1] = 10;
			stridx += 2;
		}

	header[0][stridx] = '#';
	header[0][stridx+1] = '^';
	header[0][stridx+2] = 13;
	header[0][stridx+3] = 10;
	
	stridx += 4;

	if (*bytes != stridx)
		{
			/* ERROR: Header size mismatch. ERROR CODE 3 */
			#ifdef _DEBUG
			fprintf (stderr, "ERROR: uni_create_header - Header size mismatch: %ld %d\n", *bytes, stridx);
			#endif
			return 3;
		}
	return 0;
}
