#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "libuni.h"

int
uni_deletekey (const char *keyname, struct unifile *ufile)
{

	int i;
	int result;
	int lines;
	char **ptr;

	lines = ufile[0].header_lines;

	if (ufile[0].header == NULL || lines < 1)
		{
			#ifdef _DEBUG
			fprintf (stderr, "ERROR: uni_deletekey.c - header empty.\n");
			#endif
			return 1;
		}

	result = uni_findkey (keyname, ufile);

	if (result < 0)
		{
			#ifdef _DEBUG
			fprintf (stderr, "ERROR: uni_deletekey.c - unable to find key: %s\n", keyname);
			#endif
			return 2;
		}

	/* Free the memory taken up by the key string */
	
	free (ufile[0].header[result]);

	for (i = result; i < lines - 1; i++)
		{
			/* We simply assign the pointer at i to i + 1 */
			ufile[0].header[i] = ufile[0].header[i+1];
		}

	/* Now we can decrement header_lines and realloc */

	if (lines > 0)
		{
			lines--;
		}

	ptr = (char **) realloc (ufile[0].header, lines*sizeof (char**));

	if (ptr == NULL)
		{
			#ifdef _DEBUG
			fprintf (stderr, "ERROR: uni_deletekey.c - realloc failed for header\n");
			#endif
			return 3;
		}

	ufile[0].header = ptr;
	ufile[0].header_lines = lines;

	/* Now, everything should be kosher! */

	return 0;

}
