/*
 * uni_parse_header.c
 *
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <assert.h>
#include "libuni.h"

/* uni_parse_header : Returns 0 on successful completion. */
int
uni_parse_header (char *header, int bytes, struct unifile *ufile)
{
	/* Index */
	int i;

	/* Current position in string */
	int stridx;

	/* Number of lines */
	int count;

	/* Used for return values */
	int result;

	/* We don't need the last 4 bytes of the header, since
	 * we know they are "#^^M"
	 */
	int limit;

	char *keyval;

	/* Start at the beginning */
	stridx = 0;
	ufile[0].header_lines = 0;

	limit = (bytes - 2);
	header[limit-2] = 13;
	header[limit-1] = 10;
	header[limit] = '\0';

	/* limit -1 because we increment within the loop */
	for (i = 0; i < limit-1; i++)
		{
			/* We loop through the header, until we find a CR-LF */
	
			if (header[i] == 13)
				{
					i++;
					if (header[i] == 10)
						{
							/* Found another line for the header */
							ufile[0].header_lines++;
						}
				}
		}

	ufile[0].header = (char **) malloc (sizeof (char *) * (ufile[0].header_lines));

	count = 0;
	stridx = 0;

	/* bytes - 1 as we refer to i+1 in our condition */
	for (i = 0; i < limit - 1; i++)
		{
			if (header[i] == 13 && header[i+1] == 10 && count < ufile[0].header_lines)
				{
					int length;
					/* Length of the current string, _sans_ null terminator */
					length = i - stridx;

					/* Replace with a '\0' - length holds the number of elements */
					ufile[0].header[count] = (char *) malloc (sizeof (char) * length + 1);

					/* strncpy adds a \0 to the end of the string */
					strncpy (ufile[0].header[count], header + stridx, length);

					/* Paranoia to ensure that the string ends in null */
					ufile[0].header[count][length] = '\0';

					/* i+2 because i+1 is the null character. */
					stridx = i+2;
					
					/* i gets incremented again at the top of the loop */
					i++;

					/* We added another line */
					count++;
				}
		}

	/* Now, we need to look for reserved keys, get their values, and prune 
	 * them from the user header! */
	
	result = uni_findkey ("X1,DX", ufile);

	if (result == -1)
		{
			/* Unable to find key... old version? */
			result = uni_findkey ("WVL1,DWVL", ufile);

			if (result == -1)
				{
					#ifdef _DEBUG
					fprintf (stderr, "ERROR: uni_parse_header.c - Unable to find an X1,DX or WVL1\n");
					#endif
					return 2;
				}

			/* Got an old-style value. */
			result = uni_readkey (&keyval, "WVL1,DWVL", ufile);
			if (!result)
				{
					/* Prune reserved key from user header */
					result = result || uni_deletekey ("WVL1,DWVL", ufile);
				}
		}
	else
		{
			result = uni_readkey (&keyval, "X1,DX", ufile);
			if (!result)
				{
					/* Prune reserved key from user header */
					result = result || uni_deletekey ("X1,DX", ufile);
				}
		}

	if (result)
		{
			#ifdef _DEBUG
			fprintf (stderr, "ERROR: uni_parse_header.c - uni_readkey returned %d finding X1,DX\n", result);
			#endif
			return 3;
		}

	result = sscanf (keyval, "  %le %le", &(ufile[0].x1), &(ufile[0].dx));

	if (result != 2)
		{
			#ifdef _DEBUG
			fprintf (stderr, "ERROR: uni_parse_header.c - Some problem during sscanf for X1,DX\n");
			#endif
			return 4;
		}

	/* Done with keyval */
	free (keyval);

	/* Our header should be read in, and ready to go! */

	return 0;
}
