/*
 * uni_read_file.c
 *
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "libuni.h"

/* uni_read_file : Returns 0 on successful completion. */
int
uni_read_file (char *filename, struct unifile *ufile)
{
	int result;
	FILE *fp;
	char *header;
	long int header_bytes;

	/* Try to open the file */
	fp = fopen (filename, "rb");

	if (fp == NULL)
		{
			/* File error - ERROR Code: 1 */
			return 1;
		}

	result = uni_read_header (fp, &header, &header_bytes);

	if (result)
		{
			/* Problem with uni_read_header */
			#ifdef _DEBUG
			fprintf (stderr, "ERROR: uni_read_file.c - uni_read_header returned %d\n", result);
			#endif
			return 1;
		}

	result = uni_parse_header (header, header_bytes, ufile);

	if (result)
		{
			/* Problem with uni_parse_header */
			#ifdef _DEBUG
			fprintf (stderr, "ERROR: uni_read_file.c - uni_parse_header returned %d\n", result);
			#endif
			return 2;
		}

  /* Now it is parsed and into the unifile struct */
  free (header);

	result = uni_read_data (fp, &ufile[0].data, &ufile[0].num_elems);

	if (result)
		{
			/* Problem with uni_read_data */
			#ifdef _DEBUG
			fprintf (stderr, "ERROR: uni_read_file.c - uni_read_data returned %d\n", result);
			#endif
			return 3;
		}

	fclose (fp);

	result = uni_calc_meta (ufile);
	
	return 0;
}
